/**************************************************************************************

        MUSTUXLIB - THE COMMON LIBRARY FOR ALL MUSTUX APPLICATIONS
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/

#ifndef MUSTUXAUDIODEVICEMAPPER_HH
#define MUSTUXAUDIODEVICEMAPPER_HH

#define ALSA_PCM_NEW_HW_PARAMS_API
#include <alsa/asoundlib.h>

class Bus;

namespace MustuxAudioDeviceMapper
        {
        static const int MAX_CARDS = 10;
        static const int MAX_BUSES = 20;
        static const int FIRST_BUS_IN = -200;
        static const int FIRST_BUS_IN_AVAILABLE = -201;
        static const int FIRST_BUS_OUT = -210;
        static const int FIRST_BUS_OUT_AVAILABLE = -211;

        static const int MONO = 1; // MONO mode
        static const int STEREO = 2; // STEREO mode
        static const int ONLY_LEFT_CHANNEL = 3; // STEREO mode (but uses only LEFT)
        static const int ONLY_RIGHT_CHANNEL = 4; // STEREO mode (but uses only RIGHT)

        static const int PLAYBACK = 0;
        static const int CAPTURE = 1;
        static const int MINIMUM_RATE = 44100;
        static const int MINIMUM_BIT_DEPTH = 16;
        static const int STANDARD_FPS = 10;

        static const int ERROR_INVALID_CARD = -10;
        static const int ERROR_INVALID_BUS = -11;
        static const int ERROR_INVALID_RATE_CHANNELS_OR_BITDEPTH = -12;
        static const int ERROR_BUSY_BUS = -13;
        static const int ERROR_CANNOT_INIT_MAPPER = -14;
        static const int ERROR_CANNOT_OPEN_PLAYBACK_BUS = -15;
        static const int ERROR_CANNOT_OPEN_CAPTURE_BUS = -16;
        static const int ERROR_CANNOT_OPEN_BUS = -17;
        static const int ERROR_MONO_BUS = -18;
        static const int ERROR_BUS_DOES_NOT_SUPPORT_THIS_BIT_DEPTH   = -21;
        static const int ERROR_BUS_DOES_NOT_SUPPORT_THIS_CHANNELS    = -22;
        static const int ERROR_BUS_DOES_NOT_SUPPORT_THIS_SAMPLE_RATE = -23;
        static const int ERROR_8_BITS_BANNED_IN_MUSTUX_APPLICATIONS  = -25;
        static const int ERROR_BIT_DEPTH_SUPPORT_NOT_IMPLEMENTED_YET = -26;
        static const int ERROR_INVALID_BIT_DEPTH = -27;
        static const int ERROR_CANNOT_SET_PARAMS_ANY = -28;
	static const int ERROR_UNDERRUN = -29;
	static const int ERROR_OVERRUN = -30;

        /*! \brief Detect all cards, all playback and capture buses,
         *         map them into a tree, find their ALSA descriptors
         *         and check if them are available.
         *         Each bus will be assigned with a descriptor and a index.
         *         You can access the buses only by the index. If you have
         *         3 buses in and 2 out you will have buses i 0, 1,and 2 and
         *         buses in 0 and 1.
	 * \return 1 if initialization successful
         */
        int init(int pFps);

        bool is_stereo_bus_out(int busId);
        bool is_stereo_bus_in(int busId);

        bool validate_bus_out(int busId, int pRate, int pChannels, int pBitDepht);
        bool validate_bus_in(int busId, int pRate, int pChannels, int pBitDepht);

        int open_bus_in (int busId, int pRate, int pBitDepht, int whichChannels);
        int open_bus_out(int busId, int pRate, int pBitDepht, int whichChannels, bool block=true);

        char* get_bus_out_transfer_buffer(int busId, int whichChannels);
        char* get_bus_in_transfer_buffer(int busId, int whichChannels);

        int bus_out_transfer(int busId);
        int bus_in_transfer(int busId);

        int clean_bus_out_transfer_buffer(int busId);
        int clean_bus_in_transfer_buffer(int busId);

        int close_bus_in (int busId);
        int close_bus_out(int busId);

        int get_bus_out_common_transfer_size();
	int get_bus_in_common_transfer_size();

        int get_bus_out_transfer_size(int busId);
        int get_bus_in_transfer_size(int busId);

        int get_total_buses_out();
        int get_total_buses_in();

        int close_all_playback_buses();
        int close_all_capture_buses();

        int close_all_buses();

        QString get_bus_in_name(int busId);
        QString get_bus_in_full_name(int busId);
        QString get_bus_out_name(int busId);
        QString get_bus_out_full_name(int busId);
	QString get_bus_out_descriptor(int busId);
	QString get_bus_in_descriptor(int busId);

        int get_fps();
        void set_common_transfer_size(int ts);

        int get_total_cards();
        QString get_card_name(int cardId);
        QString get_card_full_name(int cardId);


        bool is_bus_allocated_for_playback(int busId);
        bool is_bus_allocated_for_capture(int busId);
        bool is_any_bus_allocated();
        QString get_error_mesg(int err);


        };

#include "MustuxAudioBus.hh"

#endif


