/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/


#ifndef MUSTUXAUDIOFILEFORMATS_HH
#define MUSTUXAUDIOFILEFORMATS_HH

#include <stdio.h>

// TODO : the base class for Praf , Wav , and so on...

class MustuxAudioFileFormat
        {
        };




/*
 * The native Protux Audio Format. It is a WAV-like format, but with
 * some reserved schema for specific Protux data
*/

class WavFile;

class PrafFile
    {
    public:
        static const int PRAF_HEADER_LABEL_SIZE = 16;
        static const int PRAF_HEADER_DESCRIPTION_SIZE = 56;
        static const int PRAF_HEADER_SIZE = 256;
        static const int PRAF_HEADER_RESERVED_SIZE = 128;

        PrafFile();
        ~PrafFile();
        int open(char* pFileName);
        int create(char* pFileName);
        int copy_to(char* pFileName);
        int read_header();
        int write_header(int pChannels, int pRate, int pBitDepth, long long pAudioSize,
                         const char*  pProjectLabel, const char*  pGroupLabel, const char*  pHardwareLabel,
                         const char*  pDescription);
        int append_samples_from_wav(WavFile* wavfile);
        int append_samples_from_buffer(char* buffer, int bufferSize);

        void show_info();
        void rewind_audio();
        void skip_header();
        char*  get_project_label();
        char*  get_group_label();
        char*  get_hardware_label();
        char*  get_description();
        int get_channels();
        int get_rate();
        int get_bit_depth();
        long long get_audio_size();
        int get_header_size();
        FILE* get_file();
        char*  get_filename();
        long long get_file_size();
        int fix_audio_size();

        int close_file();

        int read_fragment(char* frag, int size);

    private:

        FILE* file;
        char*  fileName;
        char formatLabel[4];
        int channels;
        int rate;
        int bitDepth;
        long long audioSize;
        int headerSize;
        char projectLabel[PRAF_HEADER_LABEL_SIZE];
        char groupLabel[PRAF_HEADER_LABEL_SIZE];
        char hardwareLabel[PRAF_HEADER_LABEL_SIZE];
        char description[PRAF_HEADER_DESCRIPTION_SIZE];
        char reserved[PRAF_HEADER_RESERVED_SIZE];
        };



class WavFile
    {
    public:
        WavFile();
        ~WavFile();

        int open(char* newFileName);
        int create(char* newFileName);
        int read_header();
        int write_header(int nchannels, int arate, int abitdepth);
        void show_info();

        void rewind_audio();
        void skip_header();

        FILE* get_file();

        int close_file();
        int get_header_size();
        long long get_audio_size();
        int get_channels();
        int get_rate();
        int get_bit_depth();

        int fix_audio_size();

        int append_samples_from_buffer(char* buffer, int bufferSize);
        int append_samples_from_buffer(char* buffer, int bufferSize, long long position);
        int read_fragment(char* buffer, int size);

    private:

        FILE* file;
        char* fileName;
        char audioType[4];
        long fileSize;
        char riffType[4];
        char useLess[8];
        short compressionCode;
        short channels;
        long rate;
        long  bytesPerSec;
        short blockSize;
        short bitDepth;
        int headerSize;
        long long audioSize;

    };

#endif
